#include <NDS/NDS.h>
#include <ds_misc.h>
#include "c_defs.h"
#include "dldi/fat.h"

void vblankinterrupt() {
	EMU_VBlank();
}

static void interrupthandler() {
	u32 flags=IF;
	if(flags&IRQ_VBLANK)
		vblankinterrupt();
//	if(flags&IRQ_FIFO_FULL)
//		fifointerrupt();
	VBLANK_INTR_WAIT_FLAGS|=flags;
	IF=flags;
}

void DS_init() {
	int i=0;
	swiFastSet(&i,IPC,DTCM-IPC,1);		//clear IPC area
	POWER_CR=POWER_ALL_2D|POWER_SWAP_LCDS;
	VRAM_ABCD=0x89819182;			//see notes.txt for VRAM layout
	VRAM_EFG=0x03000000;
	VRAM_HI=0x0081;
	REG_FIFO_CNT=FIFO_CNT_ENABLE | FIFO_CNT_SEND_CLEAR | FIFO_CNT_RECV_IRQ | FIFO_CNT_ERR;
	DISP_SR=DISP_VBLANK_IRQ;
	IRQ_HANDLER=interrupthandler;
	IE=IRQ_VBLANK;
	IF=~0;
	IME=IME_ENABLED;
}

void EMU_Init() {
	PPU_init();
	rescale(0xe000,-0x00060000);
	DISPLAY_CR=MODE_0_2D|0x28800000; //SCR base=605xxxx, CHR base=600xxxx, OBJ H-blank process disable
}

char *tabtxt[]={
"\
       \x8b                        \
 Input \x89 Display   File   Debug \
       \x8e\x8f_______________________",
"\
       \x8a         \x8b              \
 Input \x89 Display \x89 File   Debug \
______\x8c\x8d         \x8e\x8f_____________",
"\
                 \x8a      \x8b       \
 Input   Display \x89 File \x89 Debug \
________________\x8c\x8d      \x8e\x8f______",
"\
                        \x8a       \
 Input   Display   File \x89 Debug \
_______________________\x8c\x8d       "};

touchstring subscreentabs[]={{64*1+2,"Input"},{64*1+18,"Display"},{64*1+38,"File"},{64*1+52,"Debug"},{64*23-10,"Exit"}, {-1,0}};
int (*call_subscreen[])(void)={do_touchcontroller, adjustdisplay, savestuff, debugdump};
#define SCREENS (sizeof(call_subscreen)/4)

void arm9main() {
	int i;
	int subscreen=-1;
	int framecount=0;
	
	DS_init();
	FAT_InitFiles();
	EMU_Init();

	consoleinit();

	consoletext(64*2-32,"          nesDS ________________________________",0);
	do_rommenu();
	if(subscreen<0)
		hideconsole();
	
	while(1) {

		framecount++;
		if(debuginfo[VBLS]>59) {
			debuginfo[VBLS]-=60;
			debuginfo[FPS]=framecount;
			framecount=0;
		}
		
		if(IPC_KEYS&KEY_CLOSED)
			ARM9sleep();
				
		touch_update();
		if(subscreen>0) {
			call_subscreen[subscreen-1]();
			i=do_touchstrings(subscreentabs,0);
			if(touchstate==4) {
				if(i==SCREENS) {	//exit
					subscreen=-subscreen;
					hideconsole();
				} else if(i>=0 && i<SCREENS) {	// "tab" selected
					clearconsole();
					subscreen=i+1;
					consoletext(0,tabtxt[i],0);
				}
			}
		} else {
			if(touchstate==4) {
				clearconsole();
				subscreen=-subscreen;
				consoletext(0,tabtxt[subscreen-1],0);
				showconsole();
			}
		}
		play();
	}
}

int lastsave;
int firstsave;
int maxsaves;
void recorder_reset() {
	maxsaves=(freemem_end-freemem_start)/SAVESTATESIZE;
	lastsave=0;
	firstsave=0;
}

//run 1 NES frame with FF/REW control
void play() {
	static int framecount=0;
	int keys;
	if(joyflags&L_R_DISABLE)
		keys=0;
	else
		keys=IPC_KEYS;
		
	if(keys&KEY_L) {
		swiWaitForVBlank();
		framecount++;
		if(framecount>2) {
			framecount-=3;
			if(firstsave!=lastsave) {
				lastsave--;
				if(lastsave<0)
					lastsave=maxsaves-1;
				loadstate(freemem_start+SAVESTATESIZE*lastsave);
				EMU_Run();
			}
		}
	} else {
		if(!(keys&KEY_R))
			swiWaitForVBlank();
		EMU_Run();
		framecount++;
		if(framecount>8) {	//save state every 9th frame
			framecount-=9;
			savestate(freemem_start+SAVESTATESIZE*lastsave);
			lastsave++;
			if(lastsave>=maxsaves)
				lastsave=0;
			if(lastsave==firstsave) {
				firstsave++;
				if(firstsave>=maxsaves)
					firstsave=0;
			}
		}
	}
}