#ifdef CODE32
	#define swiDelay(wait) __asm{mov r0,wait; swi 0x030000,{r0},{},{r0-r3,r12}}
	#define swiDivide(dividend,divisor) __asm{mov r0,dividend;mov r1,divisor,swi 0x090000,{r0,r1},{},{r0-r3,r12}}
	#define swiWaitForVBlank() __asm{swi 0x050000,{},{},{r0-r3,r12}}
	#define swiFastCopy(src,dst,size) __asm{mov r0,src; mov r1,dst; mov r2,(size)/4; swi 0x0c0000,{r0,r1,r2},{},{r0-r3,r12}}
	#define swiFastSet(src,dst,size,fill) __asm{mov r0,src; mov r1,dst; mov r2,(size)/4|(fill)<<24; swi 0x0c0000,{r0,r1,r2},{},{r0-r3,r12}}	
	#define swiIntrWait(intr,wait) __asm{mov r0,wait; mov r1,intr; swi 0x040000,{r0,r1},{},{r0-r3,r12}}
	#define swiSleep() __asm{swi 0x060000,{},{},{r0-r3,r12}}
	#define swiHalt() __asm{swi 0x070000,{},{},{r0-r3,r12}}
	#define swi6() __asm{swi 0x060000,{},{},{r0-r3,r12}}	//sleep/halt ambiguity between libnds :(
	#define swi7() __asm{swi 0x070000,{},{},{r0-r3,r12}}
	#define swiLZ77Vram(src,dst,table) __asm{mov r0,src;mov r1,dst;mov r3,table;swi 0x120000,{r0,r1,r3},{},{r0-r3,r12}}
	#define swiChangeSoundBias(updown,delay) __asm{mov r0,updown;mov r1,delay;swi 0x080000,{r0,r1},{},{r0-r3,r12}}
	inline u16 swiCRC(u16 crc,u32 *dst,u32 size) {
		u16 result;
		__asm {
			mov r0,crc;
			mov r1,dst;
			mov r2,size;
			swi 0x0e0000,{r0-r2},{r0},{r1-r3,r12};
			mov result,r0;
		}
		return result;
	}
#endif
#ifdef CODE16
	#define swiFastSet(src,dst,size,fill) __asm{mov r0,src; mov r1,dst; mov r2,(size)/4|(fill)<<24; swi 0x0c,{r0,r1,r2},{},{r0-r3,r12}}
	#define swiSoundBias(onoff,delay) __asm{mov r0,onoff; mov r1,delay; swi 0x08,{r0-r1},{},{r0-r3,r12}}
	#define swiSleep() __asm{swi 0x06,{},{},{r0-r3,r12}}
	#define swiHalt() __asm{swi 0x07,{},{},{r0-r3,r12}}
	#define swiWaitForVBlank() __asm{swi 0x05,{},{},{r0-r3,r12}}
	#define swiIntrWait(intr,wait) __asm{mov r0,wait; mov r1,intr; swi 0x04,{r0,r1},{},{r0-r3,r12}}
	#define swiDelay(wait) __asm{mov r0,wait; swi 0x03,{r0},{},{r0-r3,r12}}
	inline u16 swiCRC(u16 crc,u32 *dst,u32 size) {
		u16 result;
		__asm {
			mov r0,crc;
			mov r1,dst;
			mov r2,size;
			swi 0x0e,{r0-r2},{r0},{r1-r3,r12};
			mov result,r0;
		}
		return result;
	}
#endif

#define swap(a,b) {__asm{swp a,a,[b]}}
#define vramSetBanks(a,b,c,d) (VRAM_CR=0x80808080|(a)|(b<<8)|(c<<16)|(d<<24))

//u8 LZread(u8 *addr) {return *addr;}
//u32 LZsize(u32 *addr) {return *addr;}
//void *LZtbl[]={(void*)LZsize,0,(void*)LZread};

////////////////////////////
// SPI
////////////////////////////

#define SPI_EN		0x8000
#define SPI_I		0x4000
#define SPI_BUSY	0x0080
#define SPI_SEL		0x0800
#define SPI_POWER	0
#define SPI_FW		0x0100
#define SPI_TOUCH	0x0200
#define SPI_MIC		0x0300

#define PM_SOUND		0x01
#define PM_BOTTOMLIGHT	0x04
#define PM_TOPLIGHT		0x08
#define PM_LED_SLOW		0x10
#define PM_LED_FAST		0x30
#define PM_OFF			0x40

/////////////////////////
//FIFO
//////////////////////////

#define IRQ_FIFO_SEND 0x20000
#define IRQ_FIFO_RECV 0x40000

#define FIFO_INTR				*(vu16*)0x4000180
#define REG_FIFO_CNT            *(vu16*)0x4000184
#define REG_FIFO_SEND           *(vu32*)0x4000188
#define REG_FIFO_RECV           *(vu32*)0x4100000

#define FIFO_INTR_I                  0x4000
#define FIFO_INTR_IREQ               0x2000
#define FIFO_INTR_STATUS_OUT         0x0f00
#define FIFO_INTR_STATUS_IN          0x000f

#define FIFO_CNT_ENABLE		0x8000
#define FIFO_CNT_ERR		0x4000
#define FIFO_CNT_RECV_IRQ	0x0400
#define FIFO_CNT_RECV_FULL	0x0200
#define FIFO_CNT_RECV_EMPTY	0x0100
#define FIFO_CNT_SEND_CLEAR	0x0008
#define FIFO_CNT_SEND_IRQ	0x0004
#define FIFO_CNT_SEND_FULL	0x0002
#define FIFO_CNT_SEND_EMPTY	0x0001

enum errmsg {FAIL_SEND_FULL,FAIL_SEND_ERR,FAIL_RECV_EMPTY,FAIL_RECV_ERR,SUCCESS};

static inline void fifo_init() {
	REG_FIFO_CNT =	FIFO_CNT_SEND_CLEAR | FIFO_CNT_RECV_IRQ | FIFO_CNT_ENABLE | FIFO_CNT_ERR;
}
/*
int fifo_send(u32 data) {
    if (REG_FIFO_CNT & FIFO_CNT_ERR) {
		REG_FIFO_CNT = FIFO_CNT_SEND_CLEAR | FIFO_CNT_SEND_TI | FIFO_CNT_RECV_RI | FIFO_CNT_ENABLE | FIFO_CNT_ERR;
		return FAIL_SEND_ERR;
    }
	if(REG_FIFO_CNT&FIFO_CNT_SEND_FULL)
		return FAIL_SEND_FULL;
	REG_FIFO_SEND = data;
	return SUCCESS;
}

int fifo_recv (u32 *data_buf) {
    if (REG_FIFO_CNT & FIFO_CNT_ERR ) {
		REG_FIFO_CNT = FIFO_CNT_SEND_CLEAR | FIFO_CNT_SEND_TI | FIFO_CNT_RECV_RI | FIFO_CNT_ENABLE |FIFO_CNT_ERR;
		return FAIL_RECV_ERR;
    }
    if(REG_FIFO_CNT & FIFO_CNT_RECV_EMPTY)
		return FAIL_RECV_EMPTY;
	*data_buf = REG_FIFO_RECV;
	return SUCCESS;
}
*/
//////////////
// MISC
/////////////

#define EXCEPTION_HANDLER	*(u32*)0x27ffd9c

#define IWRAM_CNT (*(vu8*)0x4000247)
#define IWRAM_99 0
#define IWRAM_77 3

#ifdef ARM7
	#define IRQ_PM 0x400000
#endif

///////////////
// CARD
///////////////

#define MCCNT0 *(vu16*)0x40001a0	//card / SPI ctrl
#define MCD *(vu16*)0x40001a2		//SPI data

/////////////
// SOUND
////////////

#define SOUNDCNT 			*(vu16*)0x4000500
#define SOUND0_VOL			*(vu16*)0x4000400
#define SOUND0_CR			*(vu16*)0x4000402
#define SOUND0_SRC			*(vu32*)0x4000404
#define SOUND0_TIMER		*(vu16*)0x4000408
#define SOUND0_REP_PT		*(vu16*)0x400040a
#define SOUND0_REP_LEN		*(vu32*)0x400040c

////////////////
// TOUCH
////////////////

#define SCREEN_WIDTH   256 
#define SCREEN_HEIGHT   192 

#define TOUCH_CNTRL_X1   (*(vu8*)0x027FFCDC)	//pixel positions of the two points you click when calibrating 
#define TOUCH_CNTRL_Y1   (*(vu8*)0x027FFCDD) 
#define TOUCH_CNTRL_X2   (*(vu8*)0x027FFCE2) 
#define TOUCH_CNTRL_Y2   (*(vu8*)0x027FFCE3) 

#define TOUCH_CAL_X1   (*(vu16*)0x027FFCD8) 	//corresponding touchscreen values
#define TOUCH_CAL_Y1   (*(vu16*)0x027FFCDA) 
#define TOUCH_CAL_X2   (*(vu16*)0x027FFCDE) 
#define TOUCH_CAL_Y2   (*(vu16*)0x027FFCE0) 
