#include <nds/nds.h>
#include <ds_misc.h>
#include <string.h>
#include "c_defs.h"
#include "dldi/fat.h"
#include "dldi/dldi.h"

void writeAPU(u32 val,u32 addr) {
	u32 *buff=soundbuff;
	int i=IPC_APU_WRITE_HEAD;
	
	buff[i]=addr|(val<<16);
	i++;
	i&=255;
	if(i!=IPC_APU_WRITE_TAIL)
		IPC_APU_WRITE_HEAD=i;
}

void Sound_reset() {
	fifo_send(FIFO_APU_RESET,0);
}

int last_x, last_y;	//most recently touched coords
int touchstate;	// <2=pen up, 2=first touch, 3=pen down, 4=pen released

//update touchscreen state
void touch_update() {
	int ts=touchstate;
	if(IPC_KEYS & KEY_TOUCH) {
		last_x=IPC_TOUCH_X;
		last_y=IPC_TOUCH_Y;
		if(ts<3)
			ts++;
	} else {
		if(ts==2 || ts==3)
			ts=4;
		else
			ts=0;
	}
	touchstate=ts;
}

//print touchable strings. return string number if it's being touched, or -1 if not
int do_touchstrings(touchstring *ts, int pushstate) {
	char *str;
	int color, offset, xmin, xmax, ymin, ymax, strnum, strtouched;
	strnum=0;
	strtouched=-1;
	do {
		str=ts->str;
		offset=ts->offset;
		color=(pushstate&1) ? 0x2000:0x1000;
		pushstate>>=1;
		
		if(touchstate>1) {
			xmin=((offset&0x3f)<<2)-4;
			xmax=xmin+(strlen(str)<<3)+8;
			ymin=((offset&~0x3f)>>3)-4;
			ymax=ymin+16;
//			if(initial_x>=xmin && initial_x<xmax && initial_y>=ymin && initial_y<ymax) {
				if(last_x>=xmin && last_x<xmax && last_y>=ymin && last_y<ymax) {
					strtouched=strnum;
					color=0x2000;
//					if(touchstate==3) {
//						strtouched|=0x80000000;
//					}
				}
//			}
		}
		consoletext(offset,str,color);
		
		ts++;
		strnum++;
	} while(ts->offset>=0);
	return strtouched;
}

void load_sram() {
	FAT_FILE *f;
	
	if(!(cartflags&SRAM)) return;
	if(!active_interface) return;

	romfileext[0]='s';
	romfileext[1]='a';
	romfileext[2]='v';
	romfileext[3]=0;
	f=FAT_fopen(romfilename,"r");
	if(!f) return;
	FAT_fread((u8*)NES_SRAM,1,0x2000,f);
	FAT_fclose(f);
}

//write SRAM to device
void save_sram() {
	FAT_FILE *f;

	if(!(cartflags&SRAM)) return;
	if(!active_interface) return;

	romfileext[0]='s';
	romfileext[1]='a';
	romfileext[2]='v';
	romfileext[3]=0;
	f=FAT_fopen(romfilename,"w");
	if(!f) return;
	FAT_fwrite((u8*)NES_SRAM,1,0x2000,f);
	FAT_fclose(f);
}

void write_savestate() {
	FAT_FILE *f;
	u8 *p=(u8*)freemem_start;

	if(!active_interface) return;

	romfileext[0]='s';
	romfileext[1]='s';
	romfileext[2]=0;
	savestate((u32)p);
	f=FAT_fopen(romfilename,"w");
	if(!f) return;
	FAT_fwrite(p,1,SAVESTATESIZE,f);
	FAT_fclose(f);
	recorder_reset();
}

void read_savestate() {
	FAT_FILE *f;
	int i;
	u8 *p=(u8*)freemem_start;

	if(!active_interface) return;

	romfileext[0]='s';
	romfileext[1]='s';
	romfileext[2]=0;
	f=FAT_fopen(romfilename,"r");
	if(!f) return;
	i=FAT_fread(p,1,SAVESTATESIZE,f);
	FAT_fclose(f);
	if(i==SAVESTATESIZE)
		loadstate((u32)p);
	recorder_reset();
}

void ARM9sleep() {
	IPC_SLEEP=1;	//tell arm7 to sleep
	do { swiSleep(); } while(IPC_SLEEP);
}